﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.SearchModels;
using ConfectioneryContracts.StoragesContracts;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConfectioneryDatabaseImplement.Implements
{
	public class OrderStorage : IOrderStorage
	{
		public OrderViewModel? Delete(OrderBindingModel model)
		{
            using var context = new ConfectioneryDatabase();
            var element = context.Orders
                .FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                var deletedElement = context.Orders
                                            .Include(x => x.Pastry)
                                            .Include(x => x.Client)
                                            .Include(x => x.Implementer)
                                            .FirstOrDefault(x => x.Id == model.Id)
                                            ?.GetViewModel;
                context.Orders.Remove(element);
                context.SaveChanges();
                return deletedElement;
            }
            return null;
        }

		public OrderViewModel? GetElement(OrderSearchModel model)
		{
            if (!model.Id.HasValue)
            {
                return null;
            }

            using var context = new ConfectioneryDatabase();
            if (model.ImplementerId.HasValue && model.Status.HasValue)
            {
                return context.Orders
                              .Include(x => x.Pastry)
                              .Include(x => x.Client)
                              .Include(x => x.Implementer)
                              .FirstOrDefault(x => x.ImplementerId == model.ImplementerId && x.Status == model.Status)
                              ?.GetViewModel;
            }
            if (model.ImplementerId.HasValue)
            {
                return context.Orders
                              .Include(x => x.Pastry)
                              .Include(x => x.Client)
                              .Include(x => x.Implementer)
                              .FirstOrDefault(x => x.ImplementerId == model.ImplementerId)
                              ?.GetViewModel;
            }

            return context.Orders
                .Include(x => x.Pastry)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }

		public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
		{
            using var context = new ConfectioneryDatabase();
            if (model.Id.HasValue)
            {
                return context.Orders
                    .Include(x => x.Pastry)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.Id == model.Id)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.DateFrom != null && model.DateTo != null)
            {
                return context.Orders
                    .Include(x => x.Pastry)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Pastry)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.ClientId == model.ClientId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else if (model.ImplementerId.HasValue)
            {
                return context.Orders
                    .Include(x => x.Pastry)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Where(x => x.ImplementerId == model.ImplementerId)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            else
            {
                return context.Orders
                .Include(x => x.Pastry)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Where(x => model.Status == x.Status)
                .Select(x => x.GetViewModel)
                .ToList();
            }
        }

		public List<OrderViewModel> GetFullList()
		{
            using var context = new ConfectioneryDatabase();
            return context.Orders
                    .Include(x => x.Pastry)
                    .Include(x => x.Client)
                    .Include(x => x.Implementer)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }

		public OrderViewModel? Insert(OrderBindingModel model)
		{
            using var context = new ConfectioneryDatabase();

            var newOrder = Order.Create(context, model);
            if (newOrder == null)
            {
                return null;
            }

            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders
                         .Include(x => x.Pastry)
                         .Include(x => x.Client)
                         .Include(x => x.Implementer)
                         .FirstOrDefault(x => x.Id == newOrder.Id)
                         ?.GetViewModel;
        }

		public OrderViewModel? Update(OrderBindingModel model)
		{
            using var context = new ConfectioneryDatabase();

            var order = context.Orders.Include(x => x.Client).FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return context.Orders
                          .Include(x => x.Pastry)
                          .Include(x => x.Client)
                          .Include(x => x.Implementer)
                          .FirstOrDefault(x => x.Id == model.Id)
                          ?.GetViewModel;
        }
	}
}
