﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDataModels.Enums;
using ConfectioneryDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ConfectioneryDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        [Required]
        public int PastryId { get; set; }

        public virtual Pastry Pastry { get; set; } = new();

        [Required]
        public int ClientId { get; private set; }
        public virtual Client Client { get; private set; } = new();

        public int? ImplementerId { get; set; }
        public virtual Implementer? Implementer { get; set; }

        [Required]
        public int Count { get; set; }
        [Required]
        public double Sum { get; set; }
        [Required]
        public OrderStatus Status { get; set; }
        [Required]
        public DateTime DateCreate { get; set; }

        public DateTime? DateImplement { get; set; }

        public int Id { get; set; }

        public static Order? Create(ConfectioneryDatabase context, OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                PastryId = model.PastryId,
                Pastry = context.Pastries.FirstOrDefault(x => x.Id == model.PastryId),
                ClientId = model.ClientId,
                Client = context.Clients.FirstOrDefault(x => x.Id == model.ClientId),
                ImplementerId = model.ImplementerId,
                Implementer = context.Implementers.FirstOrDefault(x => x.Id == model.ImplementerId),
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PastryId = PastryId,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            PastryName = Pastry.PastryName,
            ImplementerId = ImplementerId,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
