﻿using ConfectioneryContracts.BindingModels;
using ConfectioneryContracts.ViewModels;
using ConfectioneryDataModels.Enums;
using ConfectioneryDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace ConfectioneryFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }

        public int PastryId { get; private set; }

        public int ClientId { get; private set; }

        public int? ImplementerId { get; private set; }

        public int Count { get; private set; }

        public double Sum { get; private set; }

        public OrderStatus Status { get; set; } = OrderStatus.Неизвестен;

        public DateTime DateCreate { get; set; } = DateTime.Now;

        public DateTime? DateImplement { get; set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order()
            {
                Id = model.Id,
                PastryId = model.PastryId,
                ClientId = model.ClientId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            var order = new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                PastryId = Convert.ToInt32(element.Element("PastryId")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                DateCreate = DateTime.ParseExact(element.Element("DateCreate")!.Value, "G", null),
            };
            DateTime.TryParse(element.Element("DateImplement")!.Value, out DateTime dateImpl);

            order.DateImplement = dateImpl;

            if (!Enum.TryParse(element.Element("Status")!.Value, out OrderStatus status))
            {
                return null;
            }
            order.Status = status;
            return order;
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }

            PastryId = model.PastryId;
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateCreate = model.DateCreate;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            PastryId = PastryId,
            ClientId = ClientId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };

        public XElement GetXElement => new("Order",
          new XAttribute("Id", Id),
          new XElement("PastryId", PastryId),
          new XElement("ClientId", ClientId),
          new XElement("ImplementerId", ImplementerId),
          new XElement("Count", Count.ToString()),
          new XElement("Sum", Sum.ToString()),
          new XElement("Status", Status.ToString()),
          new XElement("DateCreate", DateCreate.ToString()),
          new XElement("DateImplement", DateImplement.ToString()));
    }
}
